@@ -24,7 +24,9 @@ class TestIVFSearchPreassigned(unittest.TestCase):
         nprobe = 10
         k = 50
 
-        idx_gpu = faiss.GpuIndexIVFFlat(res, d, nlist)
+        config = faiss.GpuIndexIVFFlatConfig()
+        config.use_raft = False
+        idx_gpu = faiss.GpuIndexIVFFlat(res, d, nlist, faiss.METRIC_L2, config)
         idx_gpu.nprobe = nprobe
 
         rs = np.random.RandomState(567)
@@ -56,7 +58,9 @@ class TestIVFSearchPreassigned(unittest.TestCase):
         nprobe = 5
         k = 50
 
-        idx_gpu = faiss.GpuIndexIVFPQ(res, d, nlist, 4, 8)
+        config = faiss.GpuIndexIVFPQConfig()
+        config.use_raft = False
+        idx_gpu = faiss.GpuIndexIVFPQ(res, d, nlist, 4, 8, faiss.METRIC_L2, config)
         idx_gpu.nprobe = nprobe
 
         rs = np.random.RandomState(567)
@@ -136,7 +140,9 @@ class TestIVFPluggableCoarseQuantizer(unittest.TestCase):
 
         # construct a GPU index using the same trained coarse quantizer
         # from the CPU index
-        idx_gpu = faiss.GpuIndexIVFFlat(res, q, d, nlist, faiss.METRIC_L2)
+        config = faiss.GpuIndexIVFFlatConfig()
+        config.use_raft = False
+        idx_gpu = faiss.GpuIndexIVFFlat(res, q, d, nlist, faiss.METRIC_L2, config)
         assert(idx_gpu.is_trained)
         idx_gpu.add(xb)
 
@@ -150,7 +156,7 @@ class TestIVFPluggableCoarseQuantizer(unittest.TestCase):
         self.assertGreaterEqual((i_g == i_c).sum(), i_g.size * 0.9)
         self.assertTrue(np.allclose(d_g, d_c, rtol=5e-5, atol=5e-5))
 
-    def test_ivfsq_cpu_coarse(self):
+    def test_ivfsq_pu_coarse(self):
         res = faiss.StandardGpuResources()
         d = 128
         nb = 5000
@@ -189,7 +195,7 @@ class TestIVFPluggableCoarseQuantizer(unittest.TestCase):
 
         self.assertGreaterEqual(knn_intersection_measure(i_c, i_g), 0.9)
 
-        self.assertTrue(np.allclose(d_g, d_c, rtol=5e-5, atol=5e-5))
+        self.assertTrue(np.allclose(d_g, d_c, rtol=2e-4, atol=2e-4))
 
     def test_ivfpq_cpu_coarse(self):
         res = faiss.StandardGpuResources()
@@ -226,8 +232,10 @@ class TestIVFPluggableCoarseQuantizer(unittest.TestCase):
 
         # construct a GPU index using the same trained coarse quantizer
         # from the CPU index
+        config = faiss.GpuIndexIVFPQConfig()
+        config.use_raft = False
         idx_gpu = faiss.GpuIndexIVFPQ(
-            res, idx_coarse_cpu, d, nlist_lvl_2, 4, 8)
+            res, idx_coarse_cpu, d, nlist_lvl_2, 4, 8, faiss.METRIC_L2, config)
         assert(not idx_gpu.is_trained)
 
         idx_gpu.train(xb)
@@ -406,6 +414,7 @@ class TestIVFIndices(unittest.TestCase):
 
         # Store values using 32-bit indices instead
         config.indicesOptions = faiss.INDICES_32_BIT
+        config.use_raft = False
         idx = faiss.GpuIndexIVFFlat(res, d, nlist, faiss.METRIC_L2, config)
         idx.train(xb)
         idx.add_with_ids(xb, xb_indices)
@@ -430,6 +439,7 @@ class TestIVFIndices(unittest.TestCase):
         xb_indices = (xb_indices_base + 4294967296).astype('int64')
 
         config = faiss.GpuIndexIVFPQConfig()
+        config.use_raft = False
         idx = faiss.GpuIndexIVFPQ(res, d, nlist, M, nbits,
                                   faiss.METRIC_L2, config)
         idx.train(xb)
@@ -490,7 +500,9 @@ class TestSQ_to_gpu(unittest.TestCase):
         res = faiss.StandardGpuResources()
         index = faiss.index_factory(32, "SQfp16")
         index.add(np.random.rand(1000, 32).astype(np.float32))
-        gpu_index = faiss.index_cpu_to_gpu(res, 0, index)
+        config = faiss.GpuClonerOptions()
+        config.use_raft = False
+        gpu_index = faiss.index_cpu_to_gpu(res, 0, index, config)
         self.assertIsInstance(gpu_index, faiss.GpuIndexFlat)
 
 
@@ -577,7 +589,10 @@ class TestGpuAutoTune(unittest.TestCase):
 
     def test_params(self):
         index = faiss.index_factory(32, "IVF65536_HNSW,PQ16")
-        index = faiss.index_cpu_to_gpu(faiss.StandardGpuResources(), 0, index)
+        res = faiss.StandardGpuResources()
+        options = faiss.GpuClonerOptions()
+        options.allowCpuCoarseQuantizer = True
+        index = faiss.index_cpu_to_gpu(res, 0, index, options)
         ps = faiss.GpuParameterSpace()
         ps.initialize(index)
         for i in range(ps.parameter_ranges.size()):
