@@ -18,7 +18,7 @@ class KernelTimer {
    public:
     /// Constructor starts the timer and adds an event into the current
     /// device stream
-    KernelTimer(cudaStream_t stream = 0);
+    KernelTimer(cudaStream_t stream = nullptr);
 
     /// Destructor releases event resources
     ~KernelTimer();
