@@ -31,7 +31,7 @@ struct OnDiskOneList {
 
 /** On-disk storage of inverted lists.
  *
- * The data is stored in a mmapped chunk of memory (base ptointer ptr,
+ * The data is stored in a mmapped chunk of memory (base pointer ptr,
  * size totsize). Each list is a range of memory that contains (object
  * List) that contains:
  *
@@ -101,9 +101,10 @@ struct OnDiskInvertedLists : InvertedLists {
 
     // copy all inverted lists into *this, in compact form (without
     // allocating slots)
-    size_t merge_from(
+    size_t merge_from_multiple(
             const InvertedLists** ils,
             int n_il,
+            bool shift_ids = false,
             bool verbose = false);
 
     /// same as merge_from for a single invlist
