@@ -12,6 +12,7 @@ import faiss
 from faiss.contrib import factory_tools
 from faiss.contrib import datasets
 
+
 class TestFactory(unittest.TestCase):
 
     def test_factory_1(self):
@@ -40,7 +41,6 @@ class TestFactory(unittest.TestCase):
         index = faiss.index_factory(12, "SQ8")
         assert index.code_size == 12
 
-
     def test_factory_3(self):
 
         index = faiss.index_factory(12, "IVF10,PQ4")
@@ -73,7 +73,8 @@ class TestFactory(unittest.TestCase):
     def test_factory_HNSW_newstyle(self):
         index = faiss.index_factory(12, "HNSW32,Flat")
         assert index.storage.sa_code_size() == 12 * 4
-        index = faiss.index_factory(12, "HNSW32,SQ8", faiss.METRIC_INNER_PRODUCT)
+        index = faiss.index_factory(12, "HNSW32,SQ8",
+                                    faiss.METRIC_INNER_PRODUCT)
         assert index.storage.sa_code_size() == 12
         assert index.metric_type == faiss.METRIC_INNER_PRODUCT
         index = faiss.index_factory(12, "HNSW,PQ4")
@@ -82,6 +83,9 @@ class TestFactory(unittest.TestCase):
         index = faiss.index_factory(12, "HNSW32,PQ4np")
         indexpq = faiss.downcast_index(index.storage)
         assert not indexpq.do_polysemous_training
+        index = faiss.index_factory(12, "HNSW32,PQ4x12np")
+        indexpq = faiss.downcast_index(index.storage)
+        self.assertEqual(indexpq.pq.nbits, 12)
 
     def test_factory_NSG(self):
         index = faiss.index_factory(12, "NSG64")
@@ -97,6 +101,12 @@ class TestFactory(unittest.TestCase):
         assert isinstance(index, faiss.IndexNSGFlat)
         assert index.nsg.R == 64
 
+        index = faiss.index_factory(12, "NSG64,PQ3x10")
+        assert isinstance(index, faiss.IndexNSGPQ)
+        assert index.nsg.R == 64
+        indexpq = faiss.downcast_index(index.storage)
+        self.assertEqual(indexpq.pq.nbits, 10)
+
         index = faiss.index_factory(12, "IVF65536_NSG64,Flat")
         index_nsg = faiss.downcast_index(index.quantizer)
         assert isinstance(index, faiss.IndexIVFFlat)
@@ -110,13 +120,20 @@ class TestFactory(unittest.TestCase):
         assert index.nlist == 65536 and index_nsg.nsg.R == 64
         assert index.pq.M == 2 and index.pq.nbits == 8
 
+    def test_factory_lsh(self):
+        index = faiss.index_factory(128, 'LSHrt')
+        self.assertEqual(index.nbits, 128)
+        index = faiss.index_factory(128, 'LSH16rt')
+        self.assertEqual(index.nbits, 16)
+
     def test_factory_fast_scan(self):
         index = faiss.index_factory(56, "PQ28x4fs")
         self.assertEqual(index.bbs, 32)
         self.assertEqual(index.pq.nbits, 4)
         index = faiss.index_factory(56, "PQ28x4fs_64")
         self.assertEqual(index.bbs, 64)
-        index = faiss.index_factory(56, "IVF50,PQ28x4fs_64", faiss.METRIC_INNER_PRODUCT)
+        index = faiss.index_factory(56, "IVF50,PQ28x4fs_64",
+                                    faiss.METRIC_INNER_PRODUCT)
         self.assertEqual(index.bbs, 64)
         self.assertEqual(index.nlist, 50)
         self.assertTrue(index.cp.spherical)
@@ -143,7 +160,6 @@ class TestFactory(unittest.TestCase):
         self.assertEqual(rf.pq.M, 25)
         self.assertEqual(rf.pq.nbits, 12)
 
-
     def test_parenthesis_refine_2(self):
         # Refine applies on the whole index including pre-transforms
         index = faiss.index_factory(50, "PCA32,IVF32,Flat,Refine(PQ25x12)")
@@ -223,6 +239,18 @@ class TestFactoryV2(unittest.TestCase):
         index = faiss.index_factory(123, "IVF456,Flat")
         self.assertEqual(index.__class__, faiss.IndexIVFFlat)
 
+    def test_ivf_suffix_k(self):
+        index = faiss.index_factory(123, "IVF3k,Flat")
+        self.assertEqual(index.nlist, 3072)
+
+    def test_ivf_suffix_M(self):
+        index = faiss.index_factory(123, "IVF1M,Flat")
+        self.assertEqual(index.nlist, 1024 * 1024)
+
+    def test_ivf_suffix_HNSW_M(self):
+        index = faiss.index_factory(123, "IVF1M_HNSW,Flat")
+        self.assertEqual(index.nlist, 1024 * 1024)
+
     def test_idmap(self):
         index = faiss.index_factory(123, "Flat,IDMap")
         self.assertEqual(index.__class__, faiss.IndexIDMap)
@@ -237,6 +265,19 @@ class TestFactoryV2(unittest.TestCase):
         index = faiss.downcast_index(index)
         self.assertEqual(index.__class__, faiss.IndexIDMap2)
 
+    def test_idmap_refine(self):
+        index = faiss.index_factory(8, "IDMap,PQ4x4fs,RFlat")
+        self.assertEqual(index.__class__, faiss.IndexIDMap)
+        refine_index = faiss.downcast_index(index.index)
+        self.assertEqual(refine_index.__class__, faiss.IndexRefineFlat)
+        base_index = faiss.downcast_index(refine_index.base_index)
+        self.assertEqual(base_index.__class__, faiss.IndexPQFastScan)
+
+        # Index now works with add_with_ids, but not with add
+        index.train(np.zeros((16, 8)))
+        index.add_with_ids(np.zeros((16, 8)), np.arange(16))
+        self.assertRaises(RuntimeError, index.add, np.zeros((16, 8)))
+
     def test_ivf_hnsw(self):
         index = faiss.index_factory(123, "IVF100_HNSW,Flat")
         quantizer = faiss.downcast_index(index.quantizer)
@@ -310,4 +351,4 @@ class TestIVFSpectralHashOwnership(unittest.TestCase):
         index = faiss.IndexIVFSpectralHash(faiss.IndexFlat(10), 10, 20, 10, 1)
         index.replace_vt(faiss.ITQTransform(10, 10))
         gc.collect()
-        index.vt.d_out # this should not crash
+        index.vt.d_out  # this should not crash
