@@ -17,6 +17,7 @@ from faiss.contrib.inspect_tools import get_additive_quantizer_codebooks
 
 faiss.omp_set_num_threads(4)
 
+
 def pairwise_distances(a, b):
     anorms = (a ** 2).sum(1)
     bnorms = (b ** 2).sum(1)
@@ -210,7 +211,6 @@ class TestResidualQuantizer(unittest.TestCase):
 
         # in practice RQ is often better than PQ but it does not the case here, so just check
         # that we are within some factor.
-        print(err_pq, err_rq)
         self.assertLess(err_rq, err_pq * 1.2)
 
     def test_beam_size(self):
@@ -285,13 +285,13 @@ class TestResidualQuantizer(unittest.TestCase):
 
         # verify that prefixes are the same
         for i in range(ds.nb):
-            print(i, ds.nb)
             br = faiss.BitstringReader(faiss.swig_ptr(codes[i]), rq.code_size)
             br2 = faiss.BitstringReader(faiss.swig_ptr(codes2[i]), rq2.code_size)
             self.assertEqual(br.read(rq2.tot_bits), br2.read(rq2.tot_bits))
             br3 = faiss.BitstringReader(faiss.swig_ptr(codes3[i]), rq3.code_size)
             self.assertEqual(br.read(rq3.tot_bits), br3.read(rq3.tot_bits))
 
+
 ###########################################################
 # Test index, index factory sa_encode / sa_decode
 ###########################################################
@@ -310,6 +310,7 @@ def unpack_codes(rq, packed_codes):
             codes[i, j] = br.read(nbi)
     return codes
 
+
 def retrain_AQ_codebook(index, xt):
     """ reference implementation of codebook retraining """
     rq = index.rq
@@ -319,10 +320,8 @@ def retrain_AQ_codebook(index, xt):
 
     x_decoded = index.sa_decode(codes_packed)
     MSE = ((xt - x_decoded) ** 2).sum() / n
-    print(f"Initial MSE on training set: {MSE:g}")
 
     codes = unpack_codes(index.rq, codes_packed)
-    print("ref codes", codes[0])
     codebook_offsets = faiss.vector_to_array(rq.codebook_offsets)
 
     # build sparse code matrix (represented as a dense matrix)
@@ -338,11 +337,9 @@ def retrain_AQ_codebook(index, xt):
         B, residuals, rank, singvals = np.linalg.lstsq(C, xt, rcond=None)
     else:
         import scipy.linalg
-        import pdb; pdb.set_trace()
         B, residuals, rank, singvals = scipy.linalg.lstsq(C, xt, )
 
     MSE = ((C @ B - xt) ** 2).sum() / n
-    print(f"MSE after retrainining: {MSE:g}")
 
     # replace codebook
     # faiss.copy_array_to_vector(B.astype('float32').ravel(), index.rq.codebooks)
@@ -439,7 +436,6 @@ class TestIndexResidualQuantizer(unittest.TestCase):
             index.rq.search_type,
             faiss.AdditiveQuantizer.ST_norm_qint8)
 
-
     def test_search_decompress(self):
         ds = datasets.SyntheticDataset(32, 1000, 1000, 100)
 
@@ -503,7 +499,6 @@ class TestIndexResidualQuantizer(unittest.TestCase):
         xt_decoded = ir.sa_decode(ir.sa_encode(xt))
         err_after_refined = ((xt - xt_decoded) ** 2).sum()
 
-        print(err_before, err_after_refined)
         # ref run 7474.98 / 7006.1777
         self.assertGreater(err_before, err_after_refined * 1.06)
 
@@ -643,15 +638,15 @@ class TestIVFResidualCoarseQuantizer(unittest.TestCase):
         np.testing.assert_array_almost_equal(CDref, CDnew, decimal=5)
         np.testing.assert_array_equal(CIref, CInew)
 
-    def test_norms_oom(self):
-        "check if allocating too large norms tables raises an exception"
-        index = faiss.index_factory(32, "RQ20x8")
-        try:
-            index.train(np.zeros((100, 32), dtype="float32"))
-        except RuntimeError:
-            pass # ok
-        else:
-            self.assertFalse()
+        # check that you can load the index without computing the tables
+        quantizer.set_beam_factor(2.0)
+        self.assertNotEqual(quantizer.rq.codebook_cross_products.size(), 0)
+        quantizer3 = faiss.deserialize_index(
+            faiss.serialize_index(quantizer),
+            faiss.IO_FLAG_SKIP_PRECOMPUTE_TABLE
+        )
+        self.assertEqual(quantizer3.rq.codebook_cross_products.size(), 0)
+        CD3, CI3 = quantizer3.search(ds.get_queries(), 10)
 
 
 ###########################################################
@@ -781,7 +776,6 @@ class TestIndexResidualQuantizerSearch(unittest.TestCase):
             else:
                 inter_2 = faiss.eval_intersection(I2, gt)
                 self.assertGreaterEqual(inter_ref, inter_2)
-                # print(st, inter_ref, inter_2)
 
 
 ###########################################################
@@ -814,7 +808,6 @@ class TestIVFResidualQuantizer(unittest.TestCase):
             index.nprobe = nprobe
             D, I = index.search(ds.get_queries(), 10)
             inter = faiss.eval_intersection(I, ds.get_groundtruth(10))
-            # print(st, "nprobe=", nprobe, "inter=", inter)
             inters.append(inter)
 
         # do a little I/O test
@@ -909,18 +902,13 @@ class TestIVFResidualQuantizer(unittest.TestCase):
             D, I = index.search(ds.get_queries(), 10)
             index.rq.search_type = faiss.AdditiveQuantizer.ST_LUT_nonorm
             D2, I2 = index.search(ds.get_queries(), 10)
-            # print(D[:5] - D2[:5])
-            # print(I[:5])
             np.testing.assert_array_almost_equal(D, D2, decimal=5)
             # there are many ties because the codes are so short
             self.assertLess((I != I2).sum(), I.size * 0.1)
 
             # D2, I2 = index2.search(ds.get_queries(), 10)
-            # print(D[:5])
-            # print(D2[:5])
 
             inter = faiss.eval_intersection(I, ds.get_groundtruth(10))
-            # print("nprobe=", nprobe, "inter=", inter)
             inters.append(inter)
         self.assertTrue(np.all(inters[1:4] >= inters[:3]))
 
@@ -937,9 +925,9 @@ class TestIVFResidualQuantizer(unittest.TestCase):
 
 
 def precomp_codebooks(codebooks):
-
+    M = len(codebooks)
     codebook_cross_prods = [
-        [c1 @ c2.T for c1 in codebooks] for c2 in codebooks
+        [codebooks[m1] @ codebooks[m].T for m1 in range(m)] for m in range(M)
     ]
     cent_norms = [
         (c ** 2).sum(1)
@@ -949,7 +937,7 @@ def precomp_codebooks(codebooks):
 
 
 ############################################################
-# Reference imelementation of table-based beam search
+# Reference imelementation of table-based beam search (use_beam_LUT=1)
 ############################################################
 
 def beam_search_encode_step_tab(codes, L, distances, codebook_cross_prods_i,
@@ -979,8 +967,6 @@ def beam_search_encode_step_tab(codes, L, distances, codebook_cross_prods_i,
             for b in range(beam_size):
                 dotprods[i, b, :] += cb[codes[i, b, j]]
 
-    # print("dps", dotprods[:3, :2, :4])
-
     new_distances += 2 * dotprods
     cent_distances = new_distances
 
@@ -1100,28 +1086,29 @@ class TestCrossCodebookComputations(unittest.TestCase):
         precomp = precomp_codebooks(codebooks)
         codebook_cross_prods_ref, cent_norms_ref = precomp
 
-        # check C++ precomp tables
-        codebook_cross_prods_ref = np.hstack([
-            np.vstack(c) for c in codebook_cross_prods_ref])
-
-        rq.compute_codebook_tables()
-        codebook_cross_prods = faiss.vector_to_array(
-            rq.codebook_cross_products)
-        codebook_cross_prods = codebook_cross_prods.reshape(
-            rq.total_codebook_size, rq.total_codebook_size)
-        cent_norms = faiss.vector_to_array(rq.cent_norms)
-
-        np.testing.assert_array_almost_equal(
-            codebook_cross_prods, codebook_cross_prods_ref, decimal=5)
-        np.testing.assert_array_almost_equal(
-            np.hstack(cent_norms_ref), cent_norms, decimal=5)
-
         # validate that the python tab-based encoding works
         xb = ds.get_database()
         ref_codes, _, _ = beam_search_encoding_ref(codebooks, xb, 7)
         new_codes, _ = beam_search_encoding_tab(codebooks, xb, 7, precomp)
         np.testing.assert_array_equal(ref_codes, new_codes)
 
+        # check C++ precomp tables
+        rq.compute_codebook_tables()
+        codebook_cross_prods = faiss.vector_to_array(
+            rq.codebook_cross_products)
+        ofs = 0
+        for m in range(1, rq.M):
+            py_table = np.vstack(codebook_cross_prods_ref[m])
+            kk = rq.codebook_offsets.at(m)
+            K = 1 << rq.nbits.at(m)
+            cpp_table = codebook_cross_prods[ofs:ofs + K * kk].reshape(kk, K)
+            ofs += kk * K
+            np.testing.assert_allclose(py_table, cpp_table, atol=1e-5)
+
+        cent_norms = faiss.vector_to_array(rq.cent_norms)
+        np.testing.assert_array_almost_equal(
+            np.hstack(cent_norms_ref), cent_norms, decimal=5)
+
         # validate the C++ beam_search_encode_step_tab function
         beam_search_encoding_tab(codebooks, xb, 7, precomp, implem="ref cpp")
 
@@ -1165,7 +1152,6 @@ class TestProductResidualQuantizer(unittest.TestCase):
         pq.train(xt)
         err_pq = eval_codec(pq, xb)
 
-        print(err_prq, err_pq)
         self.assertLess(err_prq, err_pq)
 
     def test_with_rq(self):
@@ -1186,7 +1172,6 @@ class TestProductResidualQuantizer(unittest.TestCase):
         rq.train(xt)
         err_rq = eval_codec(rq, xb)
 
-        print(err_prq, err_rq)
         self.assertEqual(err_prq, err_rq)
 
 
@@ -1270,7 +1255,6 @@ class TestIndexIVFProductResidualQuantizer(unittest.TestCase):
         """check that the error is in the same ballpark as RQ."""
         inter1 = self.eval_index_accuracy("IVF100,PRQ2x2x5_Nqint8")
         inter2 = self.eval_index_accuracy("IVF100,RQ4x5_Nqint8")
-        # print(inter1, inter2)  # 392 vs 374
         self.assertGreaterEqual(inter1 * 1.1, inter2)
 
     def test_factory(self):
